#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# Author: Qingyuan Zhang
# Date: 2021/06/15
# Version: 1.0.0
import argparse
from requests.sessions import should_bypass_proxies
from mgkegg import downloads, highlight
from mgkegg.rest import *
from mgkegg.downloads import *
from mgkegg.update import *
from mgkegg.html_parser import *
import mgkegg

def main():
    parser = argparse.ArgumentParser(prog = "mgkegg", 
    description = 'KEGG color pathway object for metagenome')
    subparsers = parser.add_subparsers(dest='subcommand')


    parser_ls = subparsers.add_parser('ls', 
    help = 'list – obtain a list of entry identifiers and associated definition')
    parser_ls.add_argument('dbentries', metavar='dbentries', nargs='+', 
    help="""This operation can be used to obtain a list of all entries in each database.  \
        <database> = pathway | brite | module | ko | genome | <org> | vg | vp | ag | compound |
        glycan | reaction | rclass | enzyme | network | variant | disease |
        drug | dgroup | organism | <medicus> """)

    parser_find = subparsers.add_parser('find', 
    help = 'find – find entries with matching query keyword or other query data')
    parser_find.add_argument('dbentries', metavar='dbentries', nargs='+', 
    help="""This is a search operation.  \
        <database> = pathway | brite | module | ko | genome | <org> | vg | vp | ag | compound |
        glycan | reaction | rclass | enzyme | network | variant | disease |
        drug | dgroup | organism | <medicus> """)
        
    parser_info = subparsers.add_parser('info', 
    help='info – display database release information and linked db information')
    parser_info.add_argument('database', metavar='database', nargs='?',
    help="""This operation displays the database release information with statistics for the databases
    <database> = kegg | pathway | brite | module | ko | genome | genes | <org> | vg | vp |
    ag | ligand | compound | glycan | reaction | rclass | enzyme | network |
    variant | disease | drug | dgroup""")

    

    parser_get = subparsers.add_parser('get', 
    help = 'find – find entries with matching query keyword or other query data')
    parser_get.add_argument('dbentries', metavar='dbentries', nargs='+',
    help = """This operation retrieves given database entries in a flat file format or in other formats
    <database> = pathway | brite | module | ko | genome | <org> | vg | vp | ag | compound |
             glycan | reaction | rclass | enzyme | network | variant | disease |
             drug | dgroup | disease_ja | drug_ja | dgroup_ja | compound_ja

    <option> = aaseq | ntseq | mol | kcf | image | conf | kgml | json""")

    parser_download = subparsers.add_parser('download',
    help = 'download - download the latest kegg reference maps with both png and html format')

    parser_update = subparsers.add_parser('update', 
    help = 'update - update to the latest kegg reference maps database')
    
    parser_highlight = subparsers.add_parser('highlight', 
    help = 'highlight - highlight the given IDs of kegg components')

    parser_highlight.add_argument('--glist', metavar='genelist', 
    help = 'a file include gene list of kegg objects')

    parser_highlight.add_argument('--outdir', '-o', 
    help='outputdir for result')

    parser_highlight.add_argument('--color', help='background highlight color')

    args = parser.parse_args()
    if args.subcommand == 'ls':
        display(['list'] + args.dbentries)
    if args.subcommand == 'find':
        display(['find'] + args.dbentries)
    if args.subcommand == 'info':
        display(['info'] + [args.database])
    if args.subcommand == 'get':
        display(['get'] + args.dbentries)
    if args.subcommand == 'download':
        print('start downloading...')
        download()
    if args.subcommand == 'update':
        print('start updating...')
        update(downloads.datadir)
    if args.subcommand == 'highlight':
        color = args.color
        gls = args.glist
        outdir = args.outdir
        if not os.path.exists(outdir):
            os.makedirs(outdir)
        highlight(gls, color, mgkegg.database, outdir)
    if not args.subcommand:
        parser.print_help()
if __name__ == '__main__':
    
    main()



import os
from shutil import copyfile
from mgkegg.highlight import *
from mgkegg import html_parser
from PIL import Image
database = os.path.join(os.path.dirname(__file__), 'data')

def highlight(gls:list, color:str, database:str, outdir:str):
    kegg_object = get_gene(gls)
    for i in os.listdir(database):
        i = os.path.join(database, i)
        if 'html' not in i:
            continue
        conf = html(i).cordinate()
        cord_rect = []
        cord_poly = []
        for x in set(conf):
            for y in kegg_object:
                if y in ','.join(x[2:]):
                    if x[0] == 'rect':
                        cord_rect.append(x[1])
                    if x[0] == 'poly':
                        cord_poly.append(x[1])
                else:
                    continue
        if len(cord_rect) == 0 and len(cord_poly) == 0:
            continue
        
        img = Image.open(i.replace('html', 'png')).convert("RGB")
        
        if len(cord_rect) > 0:
            img = highlight_rect(cord_rect, img, color)
        if len(cord_poly) > 0:
            img = highlight_poly(cord_poly, img, color)
        copyfile(i, os.path.join(outdir, os.path.basename(i)))
        img.save(os.path.join(outdir, os.path.basename(i).replace('html', 'png')))    



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

import os 
import re

class html:
    def __init__(self, url):
       self.url = url 
    def localize(self):
        mapname = os.path.basename(self.url).split('.')[0]
        with open(self.url) as inf:
            html_text = inf.read()
        with open(self.url, 'w') as outf:
            for line in html_text.split('\n'):
                if 'kegg2.html' in line:
                    line = '   <a href="https://www.kegg.jp/kegg/kegg2.html"><img align="middle" alt="KEGG" border="0" src="https://www.kegg.jp/Fig/bget/kegg3.gif" /></a>\n'
                if re.search('<img .+ usemap="#mapdata"', line):
                    line = f'<img src="{mapname}.png" name="pathwayimage" name="pathwayimage" usemap="#mapdata" border="0" />'
                href = re.compile(r'"(\/\S+)"')
                if href.search(line):
                    line = href.sub(r'https://www.kegg.jp\1', line)
                man = re.compile(r"'(\/\S+)'")
                if man.search(line):
                    line = man.sub(r"'https://www.kegg.jp\1'", line)
                line += '\n'
                outf.write(line)

    def cordinate(self):
        conf = []
        with open(self.url) as f:
            for line in f:
                    
                i = re.findall(r'<area id="\w+" shape="(\w+)" coords="(.+?)".+?title="(.+?)"', line)
                
                if len(i) > 0:
                    kegg_object = re.findall('(\w+) \(.+?\)[,]?', i[0][-1])
                    if len(kegg_object) > 0:
                        conf.append(i[0][:-1] + tuple(kegg_object))
        return conf
    
    
                           
                            
                                    


#!/usr/bin/env python3
# -*- coding: utf-8 -*-

from mgkegg.rest import *
from mgkegg.html_parser import *
import asyncio
import aiohttp
import os
import re
import time

datadir = os.path.join(os.path.dirname(__file__), "data")

async def download_png(image:str, outdir:str):
    flag = True
    retry = 0
    url = f'http://rest.kegg.jp/get/{image}/image'
    if not os.path.exists(outdir):
        os.makedirs(outdir)
    while flag:
        try:
            async with aiohttp.ClientSession() as session:
                async with session.get(url) as resp:
                    with open(os.path.join(outdir, f'{image}.png'), 'wb') as pic:
                        while True:
                            chunk = await resp.content.read(64*1024)
                            if not chunk:
                                break
                            pic.write(chunk)
            flag = False
        except Exception:
            retry += 1
            time.sleep(1)
            flag = True
            print(f'retrying connect with {url} {retry} times')        

async def download_html(pathway:str, outdir:str):
    flag = True
    retry = 0
    url = f'https://www.kegg.jp/kegg-bin/show_pathway?{pathway}.html'
    if not os.path.exists(outdir):
        os.makedirs(outdir)
    while flag:
        try:
            r = requests.get(url)
            flag = False
            with open(os.path.join(outdir, f'{pathway}.html'), 'w') as f:
                f.write(r.text)
        except Exception:
            retry += 1
            time.sleep(1)
            flag = True
            print(f'retrying connect with {url} {retry} times')

def download():
    """
Download png and html
    """
    if not os.path.exists(datadir):
        os.makedirs(datadir)
    pathway_str = get(['list', 'pathway'])
    pathway_ls =  os.path.join(datadir, 'pathway.list')
    with open(pathway_ls, 'w') as f:
        f.write(pathway_str)
    pathway_list = []
    with open(pathway_ls) as f:
        for line in f:
            pathway_list += re.findall(r'path:(map\d{5})\t', line)
    flag = True
    tasks = []
    for i in range(len(pathway_list)):
        if flag: 
            loop = asyncio.new_event_loop()
            flag = False
        tasks.append(loop.create_task(download_png(pathway_list[i], datadir)))
        tasks.append(loop.create_task(asyncio.sleep(1)))
        tasks.append(loop.create_task(download_html(pathway_list[i], datadir)))
        if i % 5 == 0:
            flag = True
            loop.run_until_complete(asyncio.gather(*tasks))
            loop.run_until_complete(asyncio.sleep(1))
            tasks = []
            loop.close()
        if i == len(pathway_list)-1 and i % 5 != 0:
            loop.run_until_complete(asyncio.gather(*tasks))
            loop.close()

    for task in asyncio.Task.all_tasks():
        task.cancel()

    for pathway in pathway_list:
        html(os.path.join(datadir, f'{pathway}.html')).localize()

    with open(os.path.join(datadir, 'pathway.info'), 'w') as f:
        f.write(get(['info', 'pathway']))                      

    




#!/usr/bin/env python3
# -*- coding: utf-8 -*-

from mgkegg.html_parser import *
from PIL import ImageColor, ImageDraw
import re

def get_gene(ls:str) -> list:
    """
    get gene and color from file
    """
    with open(ls) as f:
        return [line.strip() for line in f if line !='\n']  


def highlight_rect(coords:list, image:str, color:str):
    img = image
    for i in coords:
        if len(re.findall(r'\d+', i)) != 4:
            continue
        j = [int(x) for x in re.findall(r'\d+', i)]
        
        color1 = ImageColor.getcolor(color, 'RGBA')
        X, Y, RX, RY = j
        for x in range(X, RX):
            for y in range(Y, RY):
                if img.getpixel((x, y))[0] > 0:
                    ImageDraw.floodfill(img, xy=(x, y), value = color1)
    return img

def highlight_poly(coords:list, image:str, color:str):
    poly = []
    for i in coords:
        position = [int(x) for x in re.findall(r'\d+', i)]
        for x in range(1, len(position)):
            if x % 2 == 1:
                poly.append(tuple(position[x-1:x+1]))
        color1 = ImageColor.getcolor(color, 'RGBA')
        img = ImageDraw.ImageDraw(image)
        img.polygon(poly, color)
    return image



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
import requests
rest_url = "http://rest.kegg.jp/"
def display(args):
    """
    request from kegg rest api
    """
    
    url = rest_url + '/'.join(args)
    try:
        r = requests.get(url)
        r.raise_for_status()
    except requests.exceptions.HTTPError as err:
        raise SystemExit(err)

    print(r.text)

def get(args):
    """
    get data from kegg rest api
    """
    url = rest_url + '/'.join(args)
    try:
        r = requests.get(url)
        r.raise_for_status()
    except requests.exceptions.HTTPError as err:
        raise SystemExit(err)

    return r.text     


#!/usr/bin/env python3
# -*- coding: utf-8 -*-

from mgkegg.downloads import *
from mgkegg.rest import *
import os
import re



def update(database:str):
    """
    update kegg reference pathway database
    """

    try:
        with open (os.path.join(database, 'pathway.info'),'r') as f:
            former_text = f.read()
    except FileNotFoundError as err:
        raise SystemExit(err)

    current_version = re.findall(f'Release (.+?)/', former_text)[0]
    former_version = re.findall(f'Release (.+?)/', get(['info', 'pathway']))[0]
    if current_version != former_version:
        download()
    else:
        print('You already have the latest version of kegg reference pathway')